(function ($, Drupal, drupalSettings, once) {
  Drupal.behaviors.fingerprint_distance = {
    attach(context) {
      once('fingerprint-distance', '.fingerprint-distance', context).forEach((element) => {
        const view_id = element.dataset.viewId;
        const display_id = element.dataset.displayId;
        const data = drupalSettings?.[`${view_id}-${display_id}`]?.result_data;
        const templateFingerprintDistanceDefault = `
        <div v-if="last_fingerprint" class="alert shadow-none p-2 rounded" role="alert" :style="style_alert">
        ${Drupal.t('The <b v-html="last_fingerprint.project"></b> recorded your most recent check-in time as <b v-html="last_fingerprint.time"></b>.')}
        </div>
        <div> ${Drupal.t('Accuracy: ')} <span v-html="accuracy"></span></div>
        <table class="table table-hover">
          <thead>
          <tr>
            <th>${Drupal.t('Project')}</th>
            <th>${Drupal.t('Distance')}</th>
            <th>${Drupal.t('Action')}</th>
          </tr>
          </thead>
          <tbody>
            <template v-for="(item, index) in data">
            <tr>
              <th scope="row" v-html="item.label"></th>
              <td>{{ formatNumber(item.distance) }} {{ distance_unit }}</td>
              <td>
                <button @click="checkinHandle(item.id, item.entity_type, $event)" class="btn btn-success">Checkin</button>
              </td>
            </tr>
            </template>
          </tbody>
        </table>
        `;
        const App = {
          template: typeof templateFingerprintDistance !== 'undefined' ? templateFingerprintDistance : templateFingerprintDistanceDefault,
          data() {
            return {
              locales: navigator.language,
              accuracy: Drupal.t('Low'),
              data: [],
              last_fingerprint: null,
              style_alert: null,
              distance: drupalSettings?.[`${view_id}-${display_id}`]?.distance || 50,
              distance_unit: drupalSettings?.[`${view_id}-${display_id}`]?.distance_unit || 'meters',
            }
          },
          watch:{
            last_fingerprint(newValue, oldValue) {
              this.style_alert = this.interpolateThreeColors('#D4EDDA', '#FFF3CD', '#F8D7DA')
            }
          },
          methods: {
            async checkinHandle(id, entity_type, $event) {
              $event.target.setAttribute("disabled", "");
              await axios.post(Drupal.url(`ajax/fingerprint/${entity_type}/${id}`), {
                headers: {
                  'Content-Type': 'application/json',
                  'X-CSRF-Token': ''
                }
              }).then((response) => {
                this.last_fingerprint = response.data
              }).catch(error => {
                console.log(error)
              })
              $event.target.removeAttribute("disabled");
            },
            formatNumber(num) {
              num = num.toFixed(0);
              return num.toLocaleString(this.locales)
            },
            getLocation() {
              const successCallback = (position) => {
                if (position.coords.accuracy < 22) {
                  this.accuracy = Drupal.t("High");
                }
                else {
                  this.accuracy = Drupal.t("Low");
                }
                let result_data = [];
                data.forEach((item) => {
                  let distance = this.haversine(position.coords.latitude, position.coords.longitude, item.latitude, item.longitude)
                  if (distance <= this.distance) {
                    item.distance = distance;
                    result_data.push(item);
                  }
                })
                if (result_data.length) {
                  result_data.sort((a, b) => a.distance - b.distance)
                }
                this.data = result_data;
              };

              const errorCallback = (error) => {
                return;
              };

              const noPermission = () => {
                let linkAndroid = "<a href='https://support.google.com/android/answer/3467281?hl=" + navigator.language + "'>"+ Drupal.t('Android devices') +"</a>";
                let linkIphone = "<a href='https://support.apple.com/" + navigator.language + "/102647'>" + Drupal.t('Apple devices') + "</a>";
                this.accuracy = Drupal.t("Permission to share location has not been granted. " + linkIphone + ', ' + linkAndroid);
              };

              const noServiceLocation = () => {
                this.accuracy = Drupal.t("The browser does not support geolocation.");
              };

              let options = {
                enableHighAccuracy: true,
                maximumAge: 5000,
                timeout: 5000,
              };
              if (navigator.geolocation) {
                if (navigator.permissions) {
                  navigator.permissions.query({name:'geolocation'}).then(function(result) {
                    if (result.state === 'granted' || result.state === 'prompt') {
                      navigator.geolocation.watchPosition(successCallback, errorCallback, options);
                    }
                    else if (result.state === 'denied') {
                      noPermission()
                    }
                  });
                }
                else {
                  navigator.geolocation.watchPosition(successCallback, errorCallback, options)
                }
              }
              else {
                noServiceLocation();
              }
            },
            haversine(lat1, lon1, lat2, lon2) {
              let R = 6371000;
              switch (this.distance_unit) {
                case 'miles':
                  R = 3958.8;
                  break;

                case 'kilometers':
                  R = 6371;
                  break;

                default:
                  R = 6371000;
              }
              let dLat = this.deg2rad(lat2 - lat1);
              let dLon = this.deg2rad(lon2 - lon1);
              let a =
                Math.sin(dLat/2) * Math.sin(dLat/2) +
                Math.cos(this.deg2rad(lat1)) * Math.cos(this.deg2rad(lat2)) *
                Math.sin(dLon/2) * Math.sin(dLon/2)
              ;
              let c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a));
              let d = R * c;
              return d;
            },
            deg2rad(deg) {
              return deg * (Math.PI/180)
            },
            interpolateThreeColors(color1, color2, color3) {
              let current_date = new Date();
              let last_checkin = new Date(this.last_fingerprint.created * 1000);
              let ratio = (current_date - last_checkin) / 28800000
              ratio = Math.max(0, Math.min(1, ratio));

              let startColor, endColor, localRatio;

              if (ratio < 0.5) {
                startColor = this.hexToRgb(color1);
                endColor = this.hexToRgb(color2);
                localRatio = ratio * 2; // Chuẩn hóa về khoảng 0 - 1
              } else {
                startColor = this.hexToRgb(color2);
                endColor = this.hexToRgb(color3);
                localRatio = (ratio - 0.5) * 2; // Chuẩn hóa về khoảng 0 - 1
              }

              const interpolatedRGB = {
                r: Math.round(startColor.r + localRatio * (endColor.r - startColor.r)),
                g: Math.round(startColor.g + localRatio * (endColor.g - startColor.g)),
                b: Math.round(startColor.b + localRatio * (endColor.b - startColor.b))
              };

              return {
                backgroundColor: `rgb(${interpolatedRGB.r}, ${interpolatedRGB.g}, ${interpolatedRGB.b})`,
                borderColor: `rgb(${interpolatedRGB.r}, ${interpolatedRGB.g}, ${interpolatedRGB.b})`
              };
            },
            hexToRgb(hex) {
              hex = hex.replace(/^#/, '');
              const bigint = parseInt(hex, 16);
              return {
                r: (bigint >> 16) & 255,
                g: (bigint >> 8) & 255,
                b: bigint & 255
              };
            }
          },
          async mounted() {
            this.last_fingerprint = drupalSettings?.[`${view_id}-${display_id}`]?.last_fingerprint || null;
            this.getLocation();
          }
        };
        Vue.createApp(App).mount(element)
      });
    }
  };
}(jQuery, Drupal, drupalSettings, once));
