<?php

namespace Drupal\fingerprint\Controller;

use Drupal\Core\Cache\CacheTagsInvalidatorInterface;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Returns responses for fingerprint routes.
 */
class FingerprintController extends ControllerBase {

  /**
   * FingerprintController controller require service.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request service.
   * @param \Drupal\Core\Cache\CacheTagsInvalidatorInterface $cacheTags
   *   The cache tags service.
   */
  public function __construct(protected Request $request, protected CacheTagsInvalidatorInterface $cacheTags) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('request_stack')->getCurrentRequest(),
      $container->get('cache_tags.invalidator'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function actionFingerprint($entity_type, $entity_id) {
    $current_date = new \DateTime();
    $record = $this->entityTypeManager()->getStorage('fingerprint')->create([
      'label' => $this->t('Check-in - @name [@time]', [
        '@name' => $this->currentUser()->getDisplayName(),
        '@time' => $current_date->format('d/m/Y H:i'),
      ]),
      'type' => 'checkin',
      'reference_id' => $entity_id,
      'reference_type' => $entity_type ?? 'node',
    ]);
    if ($record->save()) {
      // Reset cache tag.
      $this->cacheTags->invalidateTags(['fingerprint_user:' . $this->currentUser()->id()]);

      $project = $record->get('reference_id')->entity;
      $time = $this->formatCheckinTime(date('Y-m-d H:i:s', $record->get('created')->value));
      foreach (['getTitle', 'label', 'getName', 'getDisplayName'] as $method) {
        if (method_exists($project, $method)) {
          $project = $project->$method();
        }
      }
      return new JsonResponse([
        'id' => $record->id(),
        'uid' => $record->getOwnerId(),
        'project' => $project,
        'time' => $time,
        'reference_type' => $record->get('reference_type')->value,
        'created' => $record->get('created')->value,
      ]);
    }
    else {
      return [];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function formatCheckinTime($checkinTime) {
    $checkin = new \DateTime($checkinTime);
    $current = new \DateTime();

    // Get today's date and yesterday's date.
    $today = $current->format('Y-m-d');
    $yesterday = $current->modify('-1 day')->format('Y-m-d');

    // Get the date of the check-in time.
    $checkinDate = $checkin->format('Y-m-d');
    $checkinTimeFormatted = $checkin->format('H:i:s');

    if ($checkinDate === $today) {
      // If it is the same day.
      return $checkinTimeFormatted;
    }
    elseif ($checkinDate === $yesterday) {
      // If it is yesterday.
      return $this->t('Yesterday @time', ['@time' => $checkinTimeFormatted]);
    }
    else {
      // For other dates.
      return $checkin->format('d/m/Y H:i:s');
    }
  }

  /**
   * Ajax json response.
   */
  public function ajaxTimeline($date = '') {
    // Current week.
    $start = strtotime('this week');
    $end = strtotime('this week +6 days');
    if (!empty($date)) {
      if (strpos($date, "-W") !== FALSE) {
        [$year, $week] = explode('-W', $date);
        $start = strtotime($year . 'W' . str_pad($week, 2, '0', STR_PAD_LEFT));
        $end = strtotime('+6 days', $start);
      }
      else {
        $month = date('Y-m', strtotime($date));
        $start = strtotime(date('Y-m-01', strtotime($month)));
        $end = strtotime(date('Y-m-t', strtotime($month)));
      }
    }
    $response = $this->getData($start, $end);
    $response['method'] = 'GET';
    $response['status'] = 200;
    return new JsonResponse($response);
  }

  /**
   * Get time data.
   *
   * {@inheritdoc}
   */
  public function getData($start, $end) {
    $groups = $result = [];
    $entityManager = $this->entityTypeManager();
    $query = $entityManager->getStorage('work_time')
      ->getQuery()
      ->accessCheck(TRUE)
      ->condition('bundle', 'fingerprint')
      ->condition('created', [$start, $end], 'BETWEEN');
    $fingerprint_ids = $query->execute();
    if ($fingerprint_ids) {
      $fingerService = $entityManager->getStorage('work_time');
      $userService = $entityManager->getStorage('user');
      foreach ($fingerprint_ids as $id) {
        $entity = $fingerService->load($id);
        $uid = $entity->getOwnerId();
        if (empty($groups[$uid])) {
          $user = $userService->load($uid);
          $groups[$uid] = [
            'id' => $uid,
            'content' => empty($user) ? $entity->get('label')->value : $user->getDisplayName(),
          ];
        }
        $hour = round($entity->get('time_total')->value / 3600);
        $result[] = [
          "id" => $entity->id() . "-start",
          "content" => empty($hour) ? 'N/A' : "$hour",
          "start" => date('Y-m-d\TH:i', $entity->get('created')->value),
          'title' => date('H:i', $entity->get('created')->value),
        ];
        if (!empty($entity->get('stopped')->value)) {
          $result[] = [
            "id" => $entity->id() . "-end",
            "content" => empty($hour) ? 'N/A' : "$hour",
            "start" => date('Y-m-d\TH:i', $entity->get('stopped')->value),
            'title' => date('H:i', $entity->get('stopped')->value),
          ];
        }
      }
    }
    return ['items' => $result, 'groups' => array_values($groups)];
  }

}
