<?php

namespace Drupal\fingerprint\Form;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\fingerprint\ProcessTimeService;
use Drupal\work_time\WorkingHours;
use Drupal\work_time\WorkTimeHoliday;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides fingerprint import attendance.
 */
class FingerprintWorkTimelineForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  protected $delimiter = ',';

  /**
   * {@inheritdoc}
   */
  protected $formatDay = 'd/m/Y';

  /**
   * {@inheritdoc}
   */
  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
    protected EntityFieldManagerInterface $entityFieldManager,
    protected EntityDisplayRepositoryInterface $entityDisplayRepository,
    protected ProcessTimeService $fingerprintProcess,
    protected WorkTimeHoliday $workTimeHoliday,
    protected WorkingHours $workingHours,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager'),
      $container->get('entity_display.repository'),
      $container->get('fingerprint.process_services'),
      $container->get('work_time.holiday'),
      $container->get('work_time.working_hours')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'fingerprint_work_timeline';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $type_filter = $form_state->get('type_filter') ?? 'month';
    $date_filter = $form_state->get('date_filter') ?? NULL;
    $map_format = ['month' => 'Y-m', 'week' => 'Y-\WW'];
    if ($date_filter) {
      $date = new \DateTime($date_filter);
    }
    else {
      $date = new \DateTime();
    }
    $str_date = $date->format($map_format[$type_filter]);

    $filter_mode = $this->workingHours->modeFilter($str_date);
    $fingerprintManage = $this->entityTypeManager->getStorage('fingerprint');
    $query = $fingerprintManage->getQuery()
      ->condition('created', [$filter_mode['start']->getTimestamp(), $filter_mode['end']->getTimestamp()], 'BETWEEN')
      ->condition('uid', NULL, 'IS NOT NULL')
      ->condition('status', 1)
      ->accessCheck(FALSE);

    $data = [];
    $data_group = [];
    $query_result = $query->execute();
    if (!empty($query_result)) {
      $fingerprint_entities = $fingerprintManage->loadMultiple($query_result);
      foreach ($fingerprint_entities as $entity) {

        if ($entity->getOwner()) {
          $data_group[$entity->getOwnerId()] = [
            'id' => $entity->getOwnerId(),
            'content' => $entity->getOwner()->getDisplayName(),
          ];

          $data[] = [
            'id' => 'fingerprint-' . $entity->id(),
            'content' => date('H:i', $entity->get('created')->value),
            'title' => date('H:i', $entity->get('created')->value),
            'group' => $entity->getOwnerId(),
            'origin_group' => $entity->getOwnerId(),
            'start' => $entity->get('created')->value * 1000,
            'origin_start' => $entity->get('created')->value * 1000,
          ];
        }
      }
    }

    $workSheetManage = $this->entityTypeManager->getStorage('work_sheet');
    $query = $workSheetManage->getQuery()
      ->condition('created', [$filter_mode['start']->getTimestamp(), $filter_mode['end']->getTimestamp()], 'BETWEEN')
      ->condition('uid', NULL, 'IS NOT NULL')
      ->condition('status', 1)
      ->accessCheck(FALSE);

    $data_sheet = [];
    $data_group_sheet = [];
    $query_result_sheet = $query->execute();
    if (!empty($query_result_sheet)) {
      $work_sheet_entities = $workSheetManage->loadMultiple($query_result_sheet);
      foreach ($work_sheet_entities as $entity) {

        $data_group_sheet[$entity->getOwnerId()] = [
          'id' => $entity->getOwnerId(),
          'content' => $entity->getOwner()->getDisplayName(),
        ];

        $data_sheet[] = [
          'id' => 'work_sheet-' . $entity->id(),
          'content' => '',
          'title' => date('H:i', $entity->get('created')->value),
          'group' => $entity->getOwnerId(),
          'origin_group' => $entity->getOwnerId(),
          'start' => $entity->get('created')->value * 1000,
          'end' => $entity->get('stopped')->value * 1000,
          'break_start' => $entity->get('break_start')->value * 1000,
          'break_end' => $entity->get('break_end')->value * 1000,
          'margin_start' => $entity->get('margin_start')->value * 1000,
          'margin_end' => $entity->get('margin_end')->value * 1000,
          'type' => 'background',
        ];
      }
    }

    $form['#attached']['library'][] = 'fingerprint/fingerprint-work-timeline';
    $form['#attached']['drupalSettings']['fingerprint_work_timeline']['filter_mode'] = $filter_mode;
    $form['#attached']['drupalSettings']['fingerprint_work_timeline']['data'] = $data;
    $form['#attached']['drupalSettings']['fingerprint_work_timeline']['data_group'] = array_values($data_group);
    $form['#attached']['drupalSettings']['fingerprint_work_timeline']['data_sheet'] = $data_sheet;
    $form['#attached']['drupalSettings']['fingerprint_work_timeline']['data_group_sheet'] = array_values($data_group_sheet);

    $form['data_work_time'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Data work time'),
      '#wrapper_attributes' => [
        'class' => ['js-hide'],
      ],
    ];

    $form['wrapper']['filter'] = [
      '#type' => 'html_tag',
      '#tag' => 'div',
    ];

    $form['wrapper']['filter']['type_filter'] = [
      '#type' => 'select',
      '#title' => $this->t('Type filter'),
      '#title_display' => 'none',
      '#options' => [
        'month' => $this->t('Month'),
        'week' => $this->t('Week'),
      ],
      '#default_value' => $form_state->get('type_filter') ?? '',
      '#wrapper_attributes' => [
        'class' => ['d-inline-block'],
        'style' => 'display:inline-block',
      ],
    ];

    $form['wrapper']['filter']['date_filter'] = [
      '#type' => 'date',
      '#title' => $this->t('Date filter'),
      '#title_display' => 'none',
      '#default_value' => $str_date,
      '#attributes' => [
        'type' => $type_filter,
        'class' => ['w-auto filter-month'],
      ],
      '#wrapper_attributes' => [
        'class' => ['d-inline-block'],
        'style' => 'display:inline-block',
      ],
    ];

    $form['wrapper']['filter']['submit_filter'] = [
      '#type' => 'actions',
      '#attributes' => [
        'class' => ['d-inline-block'],
        'style' => 'display:inline-block',
      ],
    ];

    $form['wrapper']['filter']['submit_filter']['submit'] = [
      '#type' => 'submit',
      '#button_type' => 'success',
      '#value' => $this->t('Apply'),
      '#submit' => ["::submitFilter"],
    ];

    $form['timeline'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => 'timeline',
      ],
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $data = $form_state->getValue('data_work_time');
    $data = Json::decode($data);
    $operations = [];
    foreach ($data as $item) {
      $item['label'] = $this->t('Work time fingerprint');
      $item['bundle'] = 'fingerprint';
      $item['uid'] = $item['group'];
      $item['reference_id'] = NULL;
      $item['created'] = $item['start'] / 1000;
      $item['stopped'] = $item['end'] / 1000;
      $item['time_total'] = $item['stopped'] - $item['created'];
      $item['status'] = strpos('over', $item['id']) == 0 ? 0 : 1;
      $item['payroll'] = 1;
      $operations[] = [
        '\Drupal\fingerprint\CreateWorkTimeBatch::createWorkTime',
        [$item],
      ];
    }
    $batch = [
      'title' => $this->t('Creating work time...'),
      'operations' => $operations,
      'finished' => '\Drupal\fingerprint\CreateWorkTimeBatch::createWorkTimeFinishedCallback',
    ];

    batch_set($batch);
    $this->messenger()
      ->addStatus($this->t('The fingerprint has been imported.'));
    $form_state->setRedirect('fingerprint.work_timeline');
  }

  /**
   * Submit filter.
   */
  public function submitFilter(array &$form, FormStateInterface $form_state):void {
    $type_filter = $form_state->getValue('type_filter');
    $date_filter = $form_state->getValue('date_filter');
    $form_state->set('type_filter', $type_filter);
    $form_state->set('date_filter', $date_filter);
    $form_state->setRebuild();
  }

}
