<?php

namespace Drupal\fingerprint\Plugin\rest\resource;

use Drupal\Component\Plugin\DependentPluginInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\rest\Attribute\RestResource;
use Drupal\rest\ModifiedResourceResponse;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\Routing\Route;

/**
 * Represents Api Fingerprint records as resources.
 *
 * @DCG
 * The plugin exposes key-value records as REST resources. In order to enable it
 * import the resource configuration into active configuration storage. An
 * example of such configuration can be located in the following file:
 * core/modules/rest/config/optional/rest.resource.entity.node.yml.
 * Alternatively, you can enable it through admin interface provider by REST UI
 * module.
 * @see https://www.drupal.org/project/restui
 *
 * @DCG
 * Notice that this plugin does not provide any validation for the data.
 * Consider creating custom normalizer to validate and normalize the incoming
 * data. It can be enabled in the plugin definition as follows.
 * @code
 *   serialization_class = "Drupal\foo\MyDataStructure",
 * @endcode
 *
 * @DCG
 * For entities, it is recommended to use REST resource plugin provided by
 * Drupal core.
 * @see \Drupal\rest\Plugin\rest\resource\EntityResource
 */
#[RestResource(
  id: 'api_fingerprint',
  label: new TranslatableMarkup('Api Fingerprint'),
  uri_paths: [
    'canonical' => '/api/fingerprint/{id}',
    'create' => '/api/fingerprint',
  ],
)]
class ApiFingerprintResource extends ResourceBase implements DependentPluginInterface {

  /**
   * Constructs a Drupal\rest\Plugin\rest\resource\EntityResource object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param array $serializer_formats
   *   The available serialization formats.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityManager
   *   The entity type manager.
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   The current user.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, array $serializer_formats, LoggerInterface $logger, protected EntityTypeManagerInterface $entityManager, protected AccountProxyInterface $currentUser) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('rest'),
      $container->get('entity_type.manager'),
      $container->get('current_user'),
    );
  }

  /**
   * Responds to POST requests and saves the new record.
   */
  public function post(array $data): ModifiedResourceResponse {
    $fingerprintManage = $this->entityManager->getStorage('fingerprint');
    $user = $this->entityManager->getStorage('user')->load($data['group']);
    $entity = $fingerprintManage->create([
      'label' => $this->t('@name added timekeeping for @user', [
        '@name' => $this->currentUser->getDisplayName(),
        '@user' => $user->getDisplayName(),
      ]),
      'uid' => $data['group'],
      'created' => $data['start'],
    ]);
    $entity->save();
    $data['id'] = $entity->id();
    $this->logger->notice('Created new api fingerprint record @id.', ['@id' => $data['id']]);
    // Return the newly created record in the response body.
    return new ModifiedResourceResponse($data, 201);
  }

  /**
   * Responds to GET requests.
   */
  public function get($id): ResourceResponse {
    $fingerprintManege = $this->entityManager->getStorage('fingerprint');
    $entity = $fingerprintManege->load($id);
    $resource = $entity->toArray();
    return new ResourceResponse($resource);
  }

  /**
   * Responds to PATCH requests.
   */
  public function patch($id, array $data): ModifiedResourceResponse {
    $fingerprintManege = $this->entityManager->getStorage('fingerprint');
    $entity = $fingerprintManege->load($id);
    if ($entity) {
      $entity->set('created', $data['start']);
      if ($entity->save()) {
        return new ModifiedResourceResponse($data, 200);
      }
    }
    throw new NotFoundHttpException();
  }

  /**
   * Responds to DELETE requests.
   */
  public function delete($id): ModifiedResourceResponse {
    $fingerprintManege = $this->entityManager->getStorage('fingerprint');
    $entity = $fingerprintManege->load($id);
    if ($entity) {
      $entity->delete();
      $this->logger->notice('Remove fingerprint record @id.', ['@id' => $id]);
      return new ModifiedResourceResponse(NULL, 204);
    }
    throw new NotFoundHttpException();
  }

  /**
   * {@inheritdoc}
   */
  protected function getBaseRoute($canonical_path, $method): Route {
    $route = parent::getBaseRoute($canonical_path, $method);
    // Set ID validation pattern.
    if ($method !== 'POST') {
      $route->setRequirement('id', '\d+');
    }
    return $route;
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies(): array {
    return [];
  }

}
