<?php

namespace Drupal\fingerprint\Plugin\views\style;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\views\Attribute\ViewsStyle;
use Drupal\views\Plugin\views\style\DefaultStyle;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Fingerprint Distance style plugin.
 */
#[ViewsStyle(
  id: 'fingerprint_distance',
  title: new TranslatableMarkup('Fingerprint distance'),
  help: new TranslatableMarkup('Fingerprint style plugin help.'),
  theme: 'views_view_fingerprint_distance',
  display_types: ["normal"],
)]
class FingerprintDistance extends DefaultStyle {

  /**
   * {@inheritdoc}
   */
  protected $usesRowPlugin = TRUE;

  /**
   * {@inheritdoc}
   */
  protected $usesRowClass = TRUE;

  /**
   * Constructs a FingerprintDistance instance.
   *
   * @param array $configuration
   *   The configuration for the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   The entity field manager.
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   The current user.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, protected EntityTypeManagerInterface $entityTypeManager, protected EntityFieldManagerInterface $entityFieldManager, protected AccountProxyInterface $currentUser) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager'),
      $container->get('current_user'),
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['field_label'] = ['default' => 'title'];
    $options['distance'] = ['default' => '50'];
    $options['distance_unit'] = ['default' => 'meters'];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    $fields = $this->displayHandler->getHandlers('field');
    $labels = $this->displayHandler->getFieldLabels();
    $listOption = [];
    $view = $this->view;
    $view->initDisplay();
    $view->initHandlers();
    $entity_type = $this->view->getBaseEntityType()->id();
    $entity_bundles = $this->view->filter['type']->value;
    foreach ($fields as $field_name => $field) {
      foreach ($entity_bundles as $bundle) {
        $field_definition = $this->entityFieldManager->getFieldDefinitions($entity_type, $bundle)[$field->options["id"]] ?? NULL;
        if ($field_definition) {
          break;
        }
      }
      if (!empty($field_definition)) {
        $field_type_links = $field_definition->getType();
        $listOption[$field_type_links][$field_name] = $labels[$field_name];
      }
    }

    $form['field_label'] = [
      '#title' => $this->t('Field label'),
      '#type' => 'select',
      '#options' => $this->getConfigurableFields([
        'list_string',
        'text',
        'text_long',
        'text_with_summary',
        'string',
        'string_long',
      ], $listOption),
      '#default_value' => $this->options['field_label'],
      '#require' => TRUE,
    ];
    $form['field_lat_lon'] = [
      '#title' => $this->t('Field longitude latitude'),
      '#type' => 'select',
      '#options' => $this->getConfigurableFields(['geofield'], $listOption),
      '#default_value' => $this->options['field_lat_lon'],
      '#require' => TRUE,
    ];
    $form['distance'] = [
      '#title' => $this->t('Distance'),
      '#type' => 'number',
      '#default_value' => $this->options['distance'],
      '#require' => TRUE,
    ];
    $form['distance_unit'] = [
      '#type' => 'select',
      '#title' => $this->t('Unit distance'),
      "#empty_option" => $this->t("- Select -"),
      '#options' => [
        'meters' => $this->t('Meters'),
        'kilometers' => $this->t('Kilometers'),
        'miles' => $this->t('Miles'),
      ],
      '#default_value' => $this->options['distance_unit'],
      '#require' => TRUE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function preRender($results) {
    parent::preRender($results);

    if (empty($this->view->result)) {
      return;
    }
    $handler = $this->view->style_plugin;

    $option = $this->options;
    $data = [];
    foreach ($results as $delta => $result) {
      $entity = $result->_entity;
      if (!empty($option['field_lat_lon'])) {
        $field_label = $option['field_label'];
        $field_geolocation = $entity->get($option['field_lat_lon']);
        $data[] = [
          'id' => $entity->id(),
          'entity_type' => $entity->getEntityTypeId(),
          'label' => $handler->getField($delta, $field_label),
          'latitude' => $field_geolocation->lat,
          'longitude' => $field_geolocation->lon ?? $field_geolocation->lng,
          'lat_lon' => $field_geolocation->latlon,
        ];
      }
    }

    // Get last fingerprint.
    $fingerprintManage = $this->entityTypeManager->getStorage('fingerprint');
    $query = $fingerprintManage->getQuery()
      ->condition('uid', $this->currentUser->id())
      ->sort('created', 'DESC')
      ->range(0, 1)
      ->accessCheck(FALSE);

    $query_result = $query->execute();
    $last_fingerprint = NULL;
    if ($query_result) {
      $entityFingerprint = $fingerprintManage->load(reset($query_result));
      $project = $entityFingerprint->get('reference_id')->entity;
      $time = $this->formatCheckinTime(date('Y-m-d H:i:s', $entityFingerprint->get('created')->value));
      foreach (['getTitle', 'label', 'getName', 'getDisplayName'] as $method) {
        if (method_exists($project, $method)) {
          $project = $project->$method();
        }
      }
      $last_fingerprint = [
        'id' => $entityFingerprint->id(),
        'uid' => $entityFingerprint->getOwnerId(),
        'project' => $project,
        'time' => $time,
        'reference_type' => $entityFingerprint->get('reference_type')->value,
        'created' => $entityFingerprint->get('created')->value,
      ];
    }

    // Get view id.
    $view_id = $this->view->id();
    $display_id = $this->view->getDisplay()->display['id'];

    // Assign cache tag.
    $this->view->element['#cache']['tags'][] = 'fingerprint_user:' . $this->currentUser->id();

    $this->view->element['#attached']['library'][] = 'fingerprint/fingerprint-distance';
    $this->view->element["#attached"]["drupalSettings"]["$view_id-$display_id"]['last_fingerprint'] = $last_fingerprint;
    $this->view->element["#attached"]["drupalSettings"]["$view_id-$display_id"]['result_data'] = $data;
    $this->view->element["#attached"]["drupalSettings"]["$view_id-$display_id"]['distance'] = $option['distance'];
    $this->view->element["#attached"]["drupalSettings"]["$view_id-$display_id"]['distance_unit'] = $option['distance_unit'];
  }

  /**
   * Get list of fields.
   */
  protected function getConfigurableFields($type = FALSE, $listFields = []) {
    $resultField = ['' => $this->t('- None -')];
    if (!$type) {
      $resultField += $listFields;
    }
    else {
      foreach ($listFields as $field_name => $fields) {
        if (in_array($field_name, $type)) {
          $resultField += $fields;
        }
      }
    }
    return $resultField;
  }

  /**
   * {@inheritdoc}
   */
  public function formatCheckinTime($checkinTime) {
    $checkin = new \DateTime($checkinTime);
    $current = new \DateTime();

    // Get today's date and yesterday's date.
    $today = $current->format('Y-m-d');
    $yesterday = $current->modify('-1 day')->format('Y-m-d');

    // Get the date of the check-in time.
    $checkinDate = $checkin->format('Y-m-d');
    $checkinTimeFormatted = $checkin->format('H:i:s');

    if ($checkinDate === $today) {
      // If it is the same day.
      return $checkinTimeFormatted;
    }
    elseif ($checkinDate === $yesterday) {
      // If it is yesterday.
      return "Yesterday " . $checkinTimeFormatted;
    }
    else {
      // For other dates.
      return $checkin->format('d/m/Y H:i:s');
    }
  }

}
