<?php

namespace Drupal\work_sheet\Plugin\views\style;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\views\Attribute\ViewsStyle;
use Drupal\views\Plugin\views\style\StylePluginBase;
use Drupal\work_time\WorkingHours;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Yaml\Yaml;

/**
 * Work sheet schedule style plugin.
 */
#[ViewsStyle(
  id: 'work_sheet_schedule',
  title: new TranslatableMarkup('Work sheet schedule'),
  help: new TranslatableMarkup('@todo Add help text here.'),
  theme: 'views_style_work_sheet_schedule',
  display_types: ['normal'],
)]
final class WorkSheetSchedule extends StylePluginBase {

  /**
   * {@inheritdoc}
   */
  protected $usesRowPlugin = TRUE;

  /**
   * {@inheritdoc}
   */
  protected $usesRowClass = TRUE;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, protected WorkingHours $serviceWorkingHours, protected ConfigFactoryInterface $configFactory, protected EntityTypeManagerInterface $entityTypeManager, protected Request $request) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('work_time.working_hours'),
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('request_stack')->getCurrentRequest(),
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $config = $this->configFactory->get('work_time.settings');
    $options['filter'] = ['default' => $config->get('filter')];
    $options['shifts'] = ['default' => ''];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    $form['filter'] = [
      '#type' => 'select',
      '#title' => $this->t('Filter by'),
      '#description' => $this->t('Filter time total, use query example for month ?date=@month or for week or for year ?date=@year', [
        '@month' => date('Y-m'),
        '@week' => date('Y-\WW'),
      ]),
      '#options' => [
        'month' => $this->t('Month'),
        'week' => $this->t('Week'),
      ],
      '#default_value' => $this->options['filter'] ?? '',
      '#required' => TRUE,
    ];

    $workConfig = $this->configFactory->get('work_time.settings')->getRawData();

    $shifts_default = [
      'shift-office' => $workConfig['shift-office'],
      'shift' => $workConfig['shift'],
    ];

    $yaml = Yaml::dump($shifts_default, 2);

    $form['#attached']['drupalSettings']['shifts_default'] = $yaml;

    $form['shifts'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Shifts'),
      '#default_value' => $this->options['shifts'],
      '#attributes' => [
        'class' => ['yaml-editor'],
      ],
    ];
    $form['get_shifts_default'] = [
      '#type' => 'button',
      '#value' => $this->t('Get shifts default'),
      '#attributes' => [
        'class' => ['get-shifts-default'],
      ],
    ];
    $form['#attached']['library'][] = 'work_sheet/yaml_editor';
  }

  /**
   * {@inheritdoc}
   */
  public function preRender($results) {
    parent::preRender($results);
    if (!empty($this->view->total_rows)) {
      $view_id = $this->view->id();
      $display_id = $this->view->getDisplay()->display['id'];
      $this->view->element['#attached']['library'][] = 'work_sheet/work-sheet-schedule';
      $this->view->element['#attached']['drupalSettings']["$view_id-$display_id"]['views'] = [
        'view_id' => $view_id,
        'display_id' => $display_id,
      ];

      $list_entity = [];
      foreach ($results as $result) {
        $entity = $result->_entity;
        $list_entity[] = [
          'id' => $entity->id(),
          'content' => $this->getTitle($entity),
        ];
      }
      $filter = $this->request->query->get('date');
      if (empty($filter)) {
        $current_date = new \DateTime();
        $filter_mode = $this->options["filter"] ?? 'month';
        $map_format = ['month' => 'Y-m', 'week' => 'Y-\WW'];
        $filter = $current_date->format($map_format[$filter_mode]);
      }
      $modeFilter = $this->serviceWorkingHours->modeFilter($filter);

      $data_work_sheet = [];
      $workSheetManage = $this->entityTypeManager->getStorage('work_sheet');
      $query = $workSheetManage->getQuery()
        ->condition('created', [$modeFilter['start']->getTimestamp(), $modeFilter['end']->getTimestamp()], 'BETWEEN')
        ->condition('uid', array_column($list_entity, 'id'), 'IN')
        ->condition('status', 1)
        ->accessCheck(FALSE);

      $query_result = $query->execute();
      if (!empty($query_result)) {
        $work_sheet_entities = $workSheetManage->loadMultiple($query_result);
        foreach ($work_sheet_entities as $work_sheet_entity) {
          $data_work_sheet[] = [
            'id' => $work_sheet_entity->id(),
            'content' => date('H:i', $work_sheet_entity->get('created')->value),
            'title' => date('H:i', $work_sheet_entity->get('created')->value),
            'group' => $work_sheet_entity->getOwnerId(),
            'origin_group' => $work_sheet_entity->getOwnerId(),
            'start' => $work_sheet_entity->get('created')->value * 1000,
            'end' => $work_sheet_entity->get('stopped')->value * 1000,
            'origin_start' => $work_sheet_entity->get('created')->value * 1000,
            'origin_end' => $work_sheet_entity->get('stopped')->value * 1000,
            'margin_start' => $work_sheet_entity->get('margin_start')->value * 1000,
            'margin_end' => $work_sheet_entity->get('margin_end')->value * 1000,
            'type' => 'range',
          ];
        }
      }

      $shifts = [];
      $shifts_config = Yaml::parse($this->options['shifts']) ?? [];
      if (!empty($shifts_config['shift-office']) && $shifts_config['shift-office']['shift_in'] && $shifts_config['shift-office']['shift_out']) {
        $shifts[] = [
          'shift_machine_name' => 'shift-office',
          'start' => $shifts_config['shift-office']['shift_in'],
          'end' => $shifts_config['shift-office']['shift_out'],
          'margin_start' => $shifts_config['shift-office']['margin_in'] ?? NULL,
          'margin_end' => $shifts_config['shift-office']['margin_out'] ?? NULL,
          'break_start' => $shifts_config['shift-office']['break_in'] ?? NULL,
          'break_end' => $shifts_config['shift-office']['break_out'] ?? NULL,
        ];
      }

      if (!empty($shifts_config['shift'])) {
        foreach ($shifts_config['shift'] as $delta => $item_shift) {
          if (!empty($item_shift) && $item_shift['shift_in'] && $item_shift['shift_out']) {
            $shifts[] = [
              'shift_machine_name' => "shift-$delta",
              'start' => $item_shift['shift_in'],
              'end' => $item_shift['shift_out'],
              'margin_start' => $item_shift['margin_in'] ?? NULL,
              'margin_end' => $item_shift['margin_out'] ?? NULL,
              'break_start' => $item_shift['break_in'] ?? NULL,
              'break_end' => $item_shift['break_out'] ?? NULL,
            ];
          }
        }
      }

      $this->view->element['#attached']['drupalSettings']["$view_id-$display_id"]['modeFilter'] = $modeFilter;
      $this->view->element['#attached']['drupalSettings']["$view_id-$display_id"]['data'] = $list_entity;
      $this->view->element['#attached']['drupalSettings']["$view_id-$display_id"]['data_work_schedule'] = $data_work_sheet;
      $this->view->element['#attached']['drupalSettings']["$view_id-$display_id"]['data_shifts'] = $shifts;
      $this->view->element['#attached']['drupalSettings']["$view_id-$display_id"]['mode_filter'] = $modeFilter;
    }
  }

  /**
   * Get title entity.
   *
   * @param object $entity
   *   Entity.
   */
  protected function getTitle(object $entity) {
    foreach (['getTitle', 'label', 'getName', 'getDisplayName'] as $method) {
      if (method_exists($entity, $method)) {
        return $entity->$method();
      }
    }
  }

}
