<?php

namespace Drupal\work_time\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configuration form for a work time entity type.
 */
class WorkTimeConfigurationForm extends ConfigFormBase {

  /**
   * Name of the config.
   *
   * @var string
   */
  public static $configName = 'work_time.settings';

  /**
   * Constructs a Drupalist object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Config\TypedConfigManagerInterface $typedConfigManager
   *   The type config service.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   Then entity field manager service.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   The country manager service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, TypedConfigManagerInterface $typedConfigManager, protected EntityFieldManagerInterface $entityFieldManager, protected LanguageManagerInterface $languageManager) {
    parent::__construct($config_factory, $typedConfigManager);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('config.typed'),
      $container->get('entity_field.manager'),
      $container->get('language_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [self::$configName];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'work_time_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(self::$configName);
    $form['#tree'] = TRUE;
    $form['work_day'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Non working day in week'),
      '#default_value' => $config->get('work_day') ?? [],
      '#options' => array_map(fn($day) => date('l', strtotime("Sunday +$day days")), range(0, 6)),
      '#description' => $this->t('Pick out non-working days of the week'),
      '#multiple' => TRUE,
    ];
    $form['holidays'] = [
      '#title' => $this->t('General Holidays'),
      '#description' => $this->t('Enter holidays separated by, Example "01/01,01/05,09/04/2023,[22/04/2023 - 26/04/2023]... Format d/m, d/m/Y or [22/04/2023 - 26/04/2023] - Rest in between"'),
      '#type' => 'textarea',
      '#default_value' => $config->get('holidays'),
    ];
    $form['hours'] = [
      '#title' => $this->t('Total hours per day'),
      '#description' => $this->t('To determine working hours per day, For example: 8 (hours per day)'),
      '#type' => 'number',
      '#default_value' => $config->get('hours'),
    ];
    $form['over_hours'] = [
      '#title' => $this->t('Total overtime hours per day'),
      '#description' => $this->t('To defined overtime, Example: 4 (hours a day)'),
      '#type' => 'number',
      '#default_value' => $config->get('over_hours'),
    ];
    $form['filter'] = [
      '#title' => $this->t('Filter mode'),
      '#description' => $this->t('Filter default with'),
      '#type' => 'select',
      '#options' => ['week' => $this->t('Week'), 'month' => $this->t('Month')],
      '#default_value' => $config->get('filter'),
    ];
    $form['language'] = [
      '#title' => $this->t('Language'),
      '#description' => $this->t('Serves to display data in the correct format'),
      '#type' => 'select',
      '#options' => $this->listLanguages() ?? [],
      '#default_value' => $config->get('language'),
      '#require' => TRUE,
    ];

    $form['coefficient_overtime'] = [
      '#title' => $this->t('Coefficient overtime'),
      '#description' => $this->t('Coefficient overtime'),
      '#type' => 'number',
      '#step' => '0.1',
      '#default_value' => $config->get('coefficient_overtime'),
    ];

    $form['coefficient_holiday'] = [
      '#title' => $this->t('Coefficient Work in holiday'),
      '#description' => $this->t('Coefficient Work in holiday'),
      '#type' => 'number',
      '#step' => '0.1',
      '#default_value' => $config->get('coefficient_holiday'),
    ];

    $form['coefficient_day_off'] = [
      '#title' => $this->t('Coefficient Work in day off'),
      '#description' => $this->t('Coefficient Work in day off'),
      '#type' => 'number',
      '#step' => '0.1',
      '#default_value' => $config->get('coefficient_day_off'),
    ];

    $form['coefficient_vacation'] = [
      '#title' => $this->t('Coefficient Vacation'),
      '#description' => $this->t('Coefficient Vacation'),
      '#type' => 'number',
      '#step' => '0.1',
      '#default_value' => $config->get('coefficient_vacation'),
    ];

    // Office shift.
    $config_office_shift = $config->get('shift-office') ?? [];
    $form['wrapper-config-shift-office'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => [''],
        'id' => 'main-config-shift-office',
      ],
    ];

    $form['wrapper-config-shift-office']['wrapper_row_office_shift'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['row'],
      ],
    ];

    $form['wrapper-config-shift-office']['wrapper_row_office_shift']['wrapper_col_1'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['col-sm-2'],
      ],
    ];
    $form['wrapper-config-shift-office']['wrapper_row_office_shift']['wrapper_col_1']['shift_in'] = [
      '#type' => 'date',
      '#title' => $this->t('Office shift in'),
      '#default_value' => $config_office_shift['shift_in'] ?? '',
      '#attributes' => [
        'type' => 'time',
        'class' => ['form-control'],
      ],
      '#parents' => ['shift-office', 'shift_in'],
    ];

    $form['wrapper-config-shift-office']['wrapper_row_office_shift']['wrapper_col_2'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['col-sm-2'],
      ],
    ];
    $form['wrapper-config-shift-office']['wrapper_row_office_shift']['wrapper_col_2']['shift_out'] = [
      '#type' => 'date',
      '#title' => $this->t('Office shift out'),
      '#default_value' => $config_office_shift['shift_out'] ?? '',
      '#attributes' => [
        'type' => 'time',
        'class' => ['form-control'],
      ],
      '#parents' => ['shift-office', 'shift_out'],
    ];

    $form['wrapper-config-shift-office']['wrapper_row_office_shift']['wrapper_col_3'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['col-sm-2'],
      ],
    ];
    $form['wrapper-config-shift-office']['wrapper_row_office_shift']['wrapper_col_3']['margin_in'] = [
      '#type' => 'select',
      '#title' => $this->t('Delay time (after input time in minutes)'),
      '#options' => [
        '10' => $this->t('@minute minute', ['@minute' => 10]),
        '15' => $this->t('@minute minute', ['@minute' => 15]),
        '20' => $this->t('@minute minute', ['@minute' => 20]),
        '30' => $this->t('@minute minute', ['@minute' => 30]),
        '40' => $this->t('@minute minute', ['@minute' => 40]),
        '50' => $this->t('@minute minute', ['@minute' => 50]),
        '60' => $this->t('@minute minute', ['@minute' => 60]),
        '90' => $this->t('@minute minute', ['@minute' => 90]),
        '120' => $this->t('@minute minute', ['@minute' => 120]),
      ],
      '#default_value' => $config_office_shift['margin_in'] ?? '',
      '#parents' => ['shift-office', 'margin_in'],
    ];

    $form['wrapper-config-shift-office']['wrapper_row_office_shift']['wrapper_col_4'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['col-sm-2'],
      ],
    ];
    $form['wrapper-config-shift-office']['wrapper_row_office_shift']['wrapper_col_4']['margin_out'] = [
      '#type' => 'select',
      '#title' => $this->t('Time advance (before output time in minutes)'),
      '#options' => [
        '10' => $this->t('@minute minute', ['@minute' => 10]),
        '15' => $this->t('@minute minute', ['@minute' => 15]),
        '20' => $this->t('@minute minute', ['@minute' => 20]),
        '30' => $this->t('@minute minute', ['@minute' => 30]),
        '40' => $this->t('@minute minute', ['@minute' => 40]),
        '50' => $this->t('@minute minute', ['@minute' => 50]),
        '60' => $this->t('@minute minute', ['@minute' => 60]),
        '90' => $this->t('@minute minute', ['@minute' => 90]),
        '120' => $this->t('@minute minute', ['@minute' => 120]),
      ],
      '#default_value' => $config_office_shift['margin_out'] ?? '',
      '#parents' => ['shift-office', 'margin_out'],
    ];
    $form['wrapper-config-shift-office']['wrapper_row_office_shift']['wrapper_col_5'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['col-sm-2'],
      ],
    ];
    $form['wrapper-config-shift-office']['wrapper_row_office_shift']['wrapper_col_5']['break_in'] = [
      '#type' => 'date',
      '#title' => $this->t('Break in office shift'),
      '#default_value' => $config_office_shift['break_in'] ?? '',
      '#attributes' => [
        'type' => 'time',
        'class' => ['form-control'],
      ],
      '#parents' => ['shift-office', 'break_in'],
    ];
    $form['wrapper-config-shift-office']['wrapper_row_office_shift']['wrapper_col_6'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['col-sm-2'],
      ],
    ];
    $form['wrapper-config-shift-office']['wrapper_row_office_shift']['wrapper_col_6']['break_out'] = [
      '#type' => 'date',
      '#title' => $this->t('Break out office shift'),
      '#default_value' => $config_office_shift['break_out'] ?? '',
      '#attributes' => [
        'type' => 'time',
        'class' => ['form-control'],
      ],
      '#parents' => ['shift-office', 'break_out'],
    ];
    // End office shift.
    $form['wrapper-config-shift'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => [''],
        'id' => 'main-config-shift',
      ],
    ];

    $wrapper_config_shift = $config->get('shift') ?? [];
    $row_num_config_shift = 1;
    if ($wrapper_config_shift) {
      $row_num_config_shift = count($wrapper_config_shift) ?? 1;
    }

    $row_num_shift = $form_state->get('row_num_shift');
    if ($row_num_shift === NULL) {
      $form_state->set('row_num_shift', $row_num_config_shift);
      $row_num_shift = $row_num_config_shift;
    }

    for ($i = 0; $i < $row_num_shift; $i++) {
      $form['wrapper-config-shift'][$i] = [
        '#type' => 'container',
        '#attributes' => [
          'class' => ['row'],
        ],
      ];

      $form['wrapper-config-shift'][$i]['wrapper_col_1'] = [
        '#type' => 'container',
        '#attributes' => [
          'class' => ['col-sm-2'],
        ],
      ];
      $form['wrapper-config-shift'][$i]['wrapper_col_1']['shift_in'] = [
        '#type' => 'date',
        '#title' => $this->t('Shift in @number', ['@number' => $i + 1]),
        '#default_value' => $wrapper_config_shift[$i]['shift_in'] ?? '',
        '#attributes' => [
          'type' => 'time',
          'class' => ['form-control'],
        ],
        '#parents' => ['shift', $i, 'shift_in'],
      ];

      $form['wrapper-config-shift'][$i]['wrapper_col_2'] = [
        '#type' => 'container',
        '#attributes' => [
          'class' => ['col-sm-2'],
        ],
      ];
      $form['wrapper-config-shift'][$i]['wrapper_col_2']['shift_out'] = [
        '#type' => 'date',
        '#title' => $this->t('Shift out @number', ['@number' => $i + 1]),
        '#default_value' => $wrapper_config_shift[$i]['shift_out'] ?? '',
        '#attributes' => [
          'type' => 'time',
          'class' => ['form-control'],
        ],
        '#parents' => ['shift', $i, 'shift_out'],
      ];

      $form['wrapper-config-shift'][$i]['wrapper_col_3'] = [
        '#type' => 'container',
        '#attributes' => [
          'class' => ['col-sm-2'],
        ],
      ];
      $form['wrapper-config-shift'][$i]['wrapper_col_3']['margin_in'] = [
        '#type' => 'select',
        '#title' => $this->t('Delay time (after input time in minutes)'),
        '#options' => [
          '10' => $this->t('@minute minute', ['@minute' => 10]),
          '15' => $this->t('@minute minute', ['@minute' => 15]),
          '20' => $this->t('@minute minute', ['@minute' => 20]),
          '30' => $this->t('@minute minute', ['@minute' => 30]),
          '40' => $this->t('@minute minute', ['@minute' => 40]),
          '50' => $this->t('@minute minute', ['@minute' => 50]),
          '60' => $this->t('@minute minute', ['@minute' => 60]),
          '90' => $this->t('@minute minute', ['@minute' => 90]),
          '120' => $this->t('@minute minute', ['@minute' => 120]),
        ],
        '#default_value' => $wrapper_config_shift[$i]['margin_in'] ?? '',
        '#parents' => ['shift', $i, 'margin_in'],
      ];

      $form['wrapper-config-shift'][$i]['wrapper_col_4'] = [
        '#type' => 'container',
        '#attributes' => [
          'class' => ['col-sm-2'],
        ],
      ];
      $form['wrapper-config-shift'][$i]['wrapper_col_4']['margin_out'] = [
        '#type' => 'select',
        '#title' => $this->t('Time advance (before output time in minutes)'),
        '#options' => [
          '10' => $this->t('@minute minute', ['@minute' => 10]),
          '15' => $this->t('@minute minute', ['@minute' => 15]),
          '20' => $this->t('@minute minute', ['@minute' => 20]),
          '30' => $this->t('@minute minute', ['@minute' => 30]),
          '40' => $this->t('@minute minute', ['@minute' => 40]),
          '50' => $this->t('@minute minute', ['@minute' => 50]),
          '60' => $this->t('@minute minute', ['@minute' => 60]),
          '90' => $this->t('@minute minute', ['@minute' => 90]),
          '120' => $this->t('@minute minute', ['@minute' => 120]),
        ],
        '#default_value' => $wrapper_config_shift[$i]['margin_out'] ?? '',
        '#parents' => ['shift', $i, 'margin_out'],
      ];
      $form['wrapper-config-shift'][$i]['wrapper_col_5'] = [
        '#type' => 'container',
        '#attributes' => [
          'class' => ['col-sm-2'],
        ],
      ];
      $form['wrapper-config-shift'][$i]['wrapper_col_5']['break_in'] = [
        '#type' => 'date',
        '#title' => $this->t('Break in shift @number', ['@number' => $i + 1]),
        '#default_value' => $wrapper_config_shift[$i]['break_in'] ?? '',
        '#attributes' => [
          'type' => 'time',
          'class' => ['form-control'],
        ],
        '#parents' => ['shift', $i, 'break_in'],
      ];
      $form['wrapper-config-shift'][$i]['wrapper_col_6'] = [
        '#type' => 'container',
        '#attributes' => [
          'class' => ['col-sm-2'],
        ],
      ];
      $form['wrapper-config-shift'][$i]['wrapper_col_6']['break_out'] = [
        '#type' => 'date',
        '#title' => $this->t('Break out shift @number', ['@number' => $i + 1]),
        '#default_value' => $wrapper_config_shift[$i]['break_out'] ?? '',
        '#attributes' => [
          'type' => 'time',
          'class' => ['form-control'],
        ],
        '#parents' => ['shift', $i, 'break_out'],
      ];
    }
    // Button action shift.
    $form['wrapper-config-shift']['actions'] = [
      '#type' => 'actions',
    ];
    $form['wrapper-config-shift']['actions']['add_row'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add one more'),
      '#submit' => ['::addOneShift'],
      '#attrbutes' => [
        'class' => ['my-2'],
      ],
      '#ajax' => [
        'callback' => '::addMoreCallbackShift',
        'wrapper' => 'main-config-shift',
      ],
    ];
    // If there is more than one name, add the remove button.
    if ($row_num_shift > 1) {
      $form['wrapper-config-shift']['actions']['remove_name'] = [
        '#type' => 'submit',
        '#value' => $this->t('Remove one'),
        '#submit' => ['::removeCallbackShift'],
        '#ajax' => [
          'callback' => '::addMoreCallbackShift',
          'wrapper' => 'main-config-shift',
        ],
      ];
    }

    $form['actions'] = [
      '#type' => 'actions',
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Retrieve the configuration.
    $work_days = [];
    foreach ($form_state->getValue('work_day') as $item) {
      if (is_string($item)) {
        $work_days[] = $item;
      }
    }

    $shift_office = $form_state->getValue('shift-office');
    $shifts = $form_state->getValue('shift');
    foreach ($shifts as $delta => $shift) {
      if (empty($shift['shift_in']) || empty($shift['shift_out'])) {
        unset($shifts[$delta]);
      }
    }

    $this->config(self::$configName)
      // Set the submitted configuration setting.
      ->set('work_day', $work_days)
      ->set('holidays', $form_state->getValue('holidays'))
      ->set('hours', $form_state->getValue('hours'))
      ->set('over_hours', $form_state->getValue('over_hours'))
      ->set('filter', $form_state->getValue('filter'))
      ->set('coefficient_overtime', $form_state->getValue('coefficient_overtime'))
      ->set('coefficient_holiday', $form_state->getValue('coefficient_holiday'))
      ->set('coefficient_day_off', $form_state->getValue('coefficient_day_off'))
      ->set('coefficient_vacation', $form_state->getValue('coefficient_vacation'))
      ->set('language', $form_state->getValue('language'))
      ->set('shift-office', !empty($shift_office['shift_in']) && !empty($shift_office['shift_in']) ? $shift_office : [])
      ->set('shift', $shifts)
      ->save();
    $this->messenger()
      ->addStatus($this->t('The configuration has been updated.'));
  }

  /**
   * {@inheritdoc}
   */
  protected function listLanguages() {
    $languages = $this->languageManager->getStandardLanguageList() ?? [];

    $language_codes = [];
    if (!empty($languages)) {
      foreach ($languages as $code => $value) {
        $language_codes[$code] = reset($value);
      }
    }
    return $language_codes;
  }

  /**
   * Callback for both ajax-enabled buttons.
   *
   * Selects and returns the fieldset with the names in it.
   */
  public function addMoreCallbackShift(array &$form, FormStateInterface $form_state) {
    return $form['wrapper-config-shift'];
  }

  /**
   * Submit handler for the "add-one-more" button.
   *
   * Increments the max counter and causes a rebuild.
   */
  public function addOneShift(array &$form, FormStateInterface $form_state) {
    $row_num = $form_state->get('row_num_shift');
    $add_button = $row_num + 1;
    $form_state->set('row_num_shift', $add_button);
    $form_state->setRebuild();
  }

  /**
   * Submit handler for the "remove one" button.
   *
   * Decrements the max counter and causes a form rebuild.
   */
  public function removeCallbackShift(array &$form, FormStateInterface $form_state) {
    $row_num = $form_state->get('row_num_shift');
    if ($row_num > 1) {
      $remove_button = $row_num - 1;
      $form_state->set('row_num_shift', $remove_button);
    }
    $form_state->setRebuild();
  }

}
