<?php

namespace Drupal\work_time;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;

/**
 * Comment.
 */
class WorkingHours {

  use StringTranslationTrait;

  /**
   * Working hours are configurable.
   *
   * @var object
   */
  public object $workingHours;

  /**
   * Holidays are configured.
   *
   * @var array
   */
  protected array $holidays;

  /**
   * Work time report constructor.
   *
   * @param \Drupal\Core\StringTranslation\TranslationInterface $string_translation
   *   String translation.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory service.
   */
  public function __construct(TranslationInterface $string_translation, protected ConfigFactoryInterface $configFactory) {
    $this->setStringTranslation($string_translation);
    $this->workingHours = $this->configFactory->get('work_time.settings');
    $this->holidays = $this->getHolidays($this->workingHours->get('holidays') ?? '');
  }

  /**
   * Returns the date information with working hours.
   *
   * @param object $date
   *   Object date.
   *
   * @return array
   *   Is holiday?
   */
  public function isDateOff(object $date): array {
    $results = [];

    if (in_array($date->format('Y-m-d'), $this->holidays) || in_array($date->format('m-d'), $this->holidays)) {
      $results['holiday'] = TRUE;
    }
    $week_day = $date->format('w');
    $off_saturday = [];
    if (in_array($week_day, $this->workingHours->get('work_day') ?? [])) {
      $results['weekend'] = TRUE;
    }

    if (empty($results['weekend']) && $week_day == 6 && $this->workingHours->get('off_saturday')) {
      $week_number = $date->format('W');
      if ($week_number % 2) {
        $off_saturday['odd'] = TRUE;
      }
      else {
        $off_saturday['even'] = TRUE;
      }
      if (!empty($off_saturday[$this->workingHours->get('off_saturday')])) {
        $results['weekend'] = TRUE;
      }
    }
    return $results;
  }

  /**
   * Get an array of days from the input date string.
   *
   * @param string $stringDate
   *   Chuỗi thời gian, định dạng: "Y", "Y-m", "Y-\WW".
   */
  public function getRangeFilter(string $stringDate): array {
    $results = [];
    $work_day = 0;
    $weekend = 0;
    $holidays = 0;

    $filter_mode = $this->modeFilter($stringDate);
    $start = clone $filter_mode['start'];
    $end = $filter_mode['end'];
    while ($start < $end) {
      $result = ['str_date' => $start->format('Y-m-d'), 'date' => clone $start];
      $dateOff = $this->isDateOff($start);
      if (!empty($dateOff)) {
        if (!empty($dateOff['holiday'])) {
          $holidays++;
        }
        if (!empty($dateOff['weekend'])) {
          $weekend++;
        }
        if (empty($dateOff['weekend']) && !empty($dateOff['holiday'])) {
          $work_day++;
        }
      }
      else {
        $work_day++;
      }
      $result += $dateOff;
      $results['range_date'][] = $result;
      $start->modify('+1 day');
    }
    $results += [
      'work_day' => $work_day,
      'weekend' => $weekend,
      'holidays' => $holidays,
    ];
    return $results;
  }

  /**
   * Returns the type of the date, start, end filter.
   *
   * @param string $stringDate
   *   Format time: "Y", "Y-m", "Y-\WW".
   *
   * @return array
   *   Object start/end date.
   */
  public function modeFilter(string $stringDate): array {
    $date = new \DateTime($stringDate);
    $array_format = ['year' => 'Y', 'month' => 'Y-m', 'week' => 'Y-\WW'];
    $filter_mode = '';
    foreach ($array_format as $mode => $format_date) {
      if ($date->format($format_date) == $stringDate) {
        $filter_mode = $mode;
        break;
      }
    }
    switch ($filter_mode) {
      case 'year':
        $start = clone $date->modify('first day of January 00:00:00');
        $end = clone $date->modify('last day of December 23:59:59');
        break;

      case 'month':
        $start = clone $date->modify('first day of this month 00:00:00');
        $end = clone $date->modify('last day of this month 23:59:59');
        break;

      case 'week':
        $start = clone $date->modify('monday this week 00:00:00');
        $end = clone $date->modify('sunday this week 23:59:59');
        break;

      default:
        $start = $end = NULL;
    }
    return [
      'mode' => $filter_mode,
      'start' => $start,
      'end' => $end,
    ];
  }

  /**
   * Total hours worked.
   *
   * @param string $stringDate
   *   Format date: "Y", "Y-m", "Y-\WW.
   */
  public function getHoursFilter(string $stringDate): float|int {
    $range_filter = $this->getRangeFilter($stringDate);
    $hours = $this->workingHours->get('hours');
    return $hours * $range_filter['work_day'];
  }

  /**
   * Get array Holidays.
   *
   * @param string $holidays
   *   Date format "d/m", "d/m/Y".
   *
   * @return array
   *   All date holidays.
   */
  public function getHolidays(string $holidays = ''): array {
    $array_holiday = [];
    // Find all date patterns in the input string.
    if (empty($holidays)) {
      $holidays = $this->workingHours->get('holidays');
    }
    preg_match_all('/\d{2}\/\d{2}(?:\/\d{4})?/', $holidays ?? '', $matches);

    // Browse sample dates.
    if ($matches[0]) {
      foreach ($matches[0] as $val) {
        if ($check_date = $this->checkDateFormat($val)) {
          $date = DrupalDateTime::createFromFormat($check_date['format'], $val);
          $array_holiday[] = $date->format($check_date['format_to']);
        }
      }
    }

    // Extract the date ranges from the input string.
    preg_match_all('/\[(.*?)\]/', $holidays ?? '', $date_ranges);

    // Initialize the output array.
    $output_arr = [];

    // Iterate over each date range.
    foreach ($date_ranges[1] as $date_range) {
      $dates = preg_split('/[, -]+/', $date_range);
      $output_arr[] = $dates;
    }

    if (is_array($output_arr) && $output_arr) {
      foreach ($output_arr as $range) {
        if (is_array($range) && count($range) > 1) {
          $start_check = $this->checkDateFormat($range[0]);
          $end_check = $this->checkDateFormat($range[count($range) - 1]);
          if ($start_check && $end_check) {
            if ($start_check['format'] == 'd/m' || $end_check['format'] == 'd/m') {
              continue;
            }
            $start = DrupalDateTime::createFromFormat('d/m/Y', $range[0]);
            $end = DrupalDateTime::createFromFormat('d/m/Y', $range[count($range) - 1]);
            if ($start->getTimestamp() < $end->getTimestamp()) {
              while ($start->getTimestamp() <= $end->getTimestamp()) {
                $array_holiday[] = $start->format('Y-m-d');
                $start->modify('+1 day');
              }
            }
          }
        }
      }
    }

    return array_values(array_unique($array_holiday));
  }

  /**
   * Check well format d/m, d/m/Y.
   *
   * @param string $str
   *   Format date: "d/m", "d/m/Y".
   *
   * @return false|array
   *   True will return an array containing the input string.
   *   The format of the input string, the html date string.
   */
  public function checkDateFormat(string $str): false|array {
    $pattern1 = "/^(0[1-9]|[12][0-9]|3[01])\/(0[1-9]|1[012])\/(20|21)\d\d$/";
    $pattern2 = "/^(0[1-9]|[12][0-9]|3[01])\/(0[1-9]|1[012])$/";
    if (preg_match($pattern1, $str) || preg_match($pattern2, $str)) {
      $str_convert = explode('/', $str);
      switch (count($str_convert)) {
        case 2:
          $format = 'd/m';
          $formatTo = 'm-d';
          $current_date = new DrupalDateTime();
          $str_date = explode('/', $str);
          $str_date[] = $current_date->format('Y');
          $str_date = implode('/', $str_date);
          break;

        case 3:
          $format = 'd/m/Y';
          $formatTo = 'Y-m-d';
          $str_date = $str;
          break;

        default:
          return FALSE;
      }
      return [
        'date' => $str_date,
        'format' => $format,
        'format_to' => $formatTo,
      ];
    }
    else {
      return FALSE;
    }
  }

}
